@@ -35,7 +35,7 @@ class expect(object):
     To add custom matchers, just add them as instance methods to the expect object:
     
         def my_matcher(self, arguments, defaults='something):
-            pass # whatever you have to do. For helpers and availeable values see expect()
+            pass # whatever you have to do. For helpers and availeable values see dir(expect())
         expect.new_matcher = my_matcher
     
     If you want the expectation to return a bool instead of raising go for:
@@ -54,7 +54,7 @@ class expect(object):
         
         message
             can be a custom message that replaces the original message in case of error.
-            You can access the original message with the format `{assertion_message}` in 
+            You can include the original message with the format `{assertion_message}` in 
             your message. For more details see the source of self._message()
         """
         self._expected = expected
@@ -73,7 +73,7 @@ class expect(object):
         Works like the regular expect(), but instead of the error message from the matcher,
         the provided `message` will be used.
         
-        You can access the original message with the format `{assertion_message}` in 
+        You can include the original message with the format `{assertion_message}` in 
         your message. For more details see the source of self._message()
         """
         return cls(expected, message=message, should_raise=should_raise)
@@ -97,7 +97,11 @@ class expect(object):
     # All public methods on this class are expected to be matchers.
     # Beware the consequences if you break this promise. :)
     
-    # On naming matchers: 
+    # On writing matchers:
+    # You really only need to throw an AssertionError from the matcher and you're done.
+    # However, you should always use the self._assert* family of methods to do so, to support automatic negation.
+    
+    # On naming matchers:
     # Their name should be clear and fit in with the naming scheme of the existing matchers. 
     # That is: short and active if possible. Matcher names should fullfill two roles. 
     # They should allow direct usage, without chaining in front of them:
@@ -110,7 +114,7 @@ class expect(object):
     # The goal is to allow the code to read sensible and as similar to the generated error message as possible.
     # Include as many alternative names as make sense.
     
-    # Right now some be_ prefixes are included as aliasses, but I'm feeling confident that I will probably phase 
+    # Right now some be_* prefixes are included as aliasses, but I will probably phase 
     # them out in favor of using arbitrary chaining to achieve their effect.
     
     # On debugging matchers: Some pyton debuggers will hide all the internals of the expect method
@@ -134,6 +138,7 @@ class expect(object):
     
     def exist(self):
         self._assert(self._expected is not None, "to exist (not be None)")
+    
     exists = exist
     # REFACT: consider adding 'from' alias to allow syntax like expect(False).from(some_longish_expression())
     # Could enhance readability, not sure it's a good idea?
@@ -167,12 +172,12 @@ class expect(object):
     falsy = falsish = falseish
     is_falseish = falseish
     
-    def includes(self, *needles):
-        for needle in needles:
+    def includes(self, needle, *additional_needles):
+        for needle in [needle] + list(additional_needles):
             self._assert(needle in self._expected, "to include {0!r}", needle)
     
     contain = contains = include = includes
-    does_include = to_include = includes
+    does_include = to_include = has_key = includes
     
     def within(self, sequence_or_atom, *additional_atoms):
         sequence = sequence_or_atom
@@ -202,6 +207,11 @@ class expect(object):
     includes_dict = contains_dict = subdict = sub_dict
     have_subdict = have_sub_dict = has_subdict = has_sub_dict = sub_dict
     
+    def has_attribute(self, attribute_name):
+        self._assert(hasattr(self._expected, attribute_name) is True, "to have attribute {0!r}", attribute_name)
+    hasattr = has_attr = has_attribute
+    have_attribute = have_attr = has_attribute
+    
     def to_match(self, regex):
         string_type = str if sys.version > '3' else basestring
         expect(self._expected).is_instance(string_type)
@@ -256,6 +266,11 @@ class expect(object):
     isinstance = instanceof = instance_of
     is_instance = is_instance_of = instance_of
     
+    def is_subclass_of(self, a_superclass):
+        self._assert(issubclass(self._expected, a_superclass), "to be subclass of {0!r}", a_superclass)
+    
+    issubclass = is_subclass = subclass_of = subclass = is_subclass_of
+    
     def is_callable(self):
         self._assert(callable(self._expected) is True, "to be callable")
     
@@ -299,6 +314,9 @@ class expect(object):
     # TODO: consider adding is_between_exclusive
     # TODO: consider supporting slice syntax as alias. expect(3)[2:4] doesn't look natural though
     # REFACT: consider to change to be more like range(), i.e. lower bound included, upper bound excluded
+    # Alternative: add in_range that includes lower bound but excludes upper one
+    # REFACT: name could also reflect better that between actually includes both ends of the range
+    # Alternatives: in_open_range, within_half_closed_range, within_range_including_lower_bound
     def between(self, lower, higher):
         self._assert(lower <= self._expected <= higher, "to be between {0!r} and {1!r}", lower, higher)
     
@@ -337,8 +355,7 @@ class expect(object):
         if name.startswith('not_') or '_not_' in name or name.endswith('_not'):
             self._expected_assertion_result = False
         
-        matcher = self._matcher_with_name(name)
-        self._prepare_matcher_for_calling(name, matcher)
+        self._prepare_matcher_for_calling(name, self._matcher_with_name(name))
         
         # Allow arbitrary chaining
         return self
@@ -350,12 +367,12 @@ class expect(object):
         if name.startswith('not_'):
             name = name[4:]
         
-        # hasattr on __class__ to prevent recursion from __getattribute__
-        if not hasattr(self.__class__, name):
+        # need object.__getattribute__ to prevent recursion with self.__getattribute__
+        try:
+            return object.__getattribute__(self, name)
+        except AttributeError as e:
             return None
-        
-        return object.__getattribute__(self, name)
-        
+    
     def _prepare_matcher_for_calling(self, name, matcher):
         self._selected_matcher_name = name
         self._selected_matcher = matcher
@@ -372,25 +389,27 @@ class expect(object):
             # REFACT: consider raising NotImplementedError
             raise AssertionError("Tried to call non existing matcher '{0}' (Patches welcome!)".format(self._selected_matcher_name))
         
-        # Make the stacktrace easier to read by tricking python to shorten the stack trace to this method.
-        # Hides the actual matcher and all the methods it calls to assert stuff.
         try:
             return_value = self._selected_matcher(*args, **kwargs)
+            # allow an otherwise raising matcher to return something
+            # usefull for matchers like expect.to_raise() to return the caught exception for further analysis
             if self._should_raise:
-                # if not, we want to return True
                 return return_value
         except AssertionError as assertion:
-            if self._should_raise:
-                raise assertion
+            message = self._message(assertion)
             
-            # Support returning_expect
-            return (False, str(assertion))
+            if self._should_raise:
+                # Make the stacktrace easier to read by tricking python to shorten the stack trace to this method.
+                # Hides the actual matcher and all the methods it calls to assert stuff.
+                raise AssertionError(message)
+            
+            return (False, message)
         
         return (True, "")
     
     def _assert(self, assertion, message_format, *message_positionals, **message_keywords):
         assert assertion is self._expected_assertion_result, \
-            self._message(message_format, *message_positionals, **message_keywords)
+            message_format.format(*message_positionals, **message_keywords)
     
     def _assert_if_positive(self, assertion, message_format, *message_positionals, **message_keywords):
         if self._is_negative():
@@ -404,23 +423,24 @@ class expect(object):
         
         self._assert(assertion, message_format, *message_positionals, **message_keywords)
     
-    # REFACT: would be nice if the name of this method makes it clear how the 
-    # message should be worded to give a good error message
-    def _message(self, message_format, *message_positionals, **message_keywords):
+    def _message(self, assertion):
+        message = self._message_from_assertion(assertion)
         expected = self._expected
         optional_negation = ' not ' if self._is_negative() else ' '
-        message = message_format.format(*message_positionals, **message_keywords)
-        assertion_message = "Expect {expected!r}{optional_negation}{message}".format(
-            expected=expected,
-            optional_negation=optional_negation,
-            message=message,
-        )
+        assertion_message = "Expect {expected!r}{optional_negation}{message}".format(**locals())
         
         if self._custom_message is not None:
             return self._custom_message.format(**locals())
         
         return assertion_message
     
+    def _message_from_assertion(self, assertion):
+        if sys.version < '3':
+            try: return unicode(assertion).encode('utf8')
+            except UnicodeDecodeError as ignored: pass
+        
+        return str(assertion)
+    
     def _is_negative(self):
         return self._expected_assertion_result is False
     
@@ -437,23 +457,23 @@ class expect(object):
         def wrap(special_method_name, public_name):
             def wrapper(self, *args, **kwargs):
                 __tracebackhide__ = True  # Hide from py.test tracebacks
-                # Sadly this increases the traceback lenght by one entry :/
+                # Sadly this increases the traceback lenght by one entry for other testing frameworks :/
                 self.__getattribute__(public_name)(*args, **kwargs)
             setattr(cls, special_method_name, wrapper)
         
         special_names = filter(lambda name: name.startswith('__') and name.endswith('__'), dir(cls))
-        public_matchers_by_matcher = dict()
-        for name in dir(cls):
-            # Assumes that all public methods are matchers
-            if not name.startswith('_'):
-                matcher = object.__getattribute__(cls, name)
-                public_matchers_by_matcher[matcher] = name
-        public_matchers = list(public_matchers_by_matcher.keys())
+        # Assumes that all public methods are matchers
+        matcher_names = filter(lambda name: not name.startswith('_'), dir(cls))
+        public_matcher_names_by_matcher = dict()
+        for name in matcher_names:
+            matcher = getattr(cls, name)
+            public_matcher_names_by_matcher[matcher] = name
+        matchers = list(public_matcher_names_by_matcher.keys())
         
         for special_name in special_names:
-            matcher = object.__getattribute__(cls, special_name)
-            if matcher in public_matchers:
-                public_name = public_matchers_by_matcher[matcher]
+            matcher = getattr(cls, special_name)
+            if matcher in matchers:
+                public_name = public_matcher_names_by_matcher[matcher]
                 wrap(special_name, public_name)
     
 
@@ -464,28 +484,51 @@ class ExpectTest(TestCase):
     
     # Meta functionality
     
+    def test_can_subclass_expect(self):
+        """Usefull if you want to extend expect with custom matchers without polluting the original expect()
+        Used in the test suite to keep test isolation high."""
+        class local_expect(expect): pass
+        local_expect(1).equals(1)
+        expect(lambda: local_expect(1).equals(2)).to_raise(AssertionError, r"^Expect 1 to equal 2$")
+    
     def test_can_add_custom_matchers(self):
         calls = []
-        expect.custom_matcher = lambda *arguments: calls.append(arguments)
+        class local_expect(expect): pass
+        local_expect.custom_matcher = lambda *arguments: calls.append(arguments)
         
-        instance = expect('foo')
+        instance = local_expect('foo')
         instance.custom_matcher()
         instance.custom_matcher('bar', 'baz')
         
         expect(calls).to.contain((instance,))
         expect(calls).to.contain((instance, 'bar', 'baz'))
     
-    def test_good_error_message_when_calling_non_existing_matcher(self):
+    def test_assertion_error_from_matcher_is_enhanced(self):
+        def raise_(self):
+            raise AssertionError('sentinel')
+        expect.fnord = raise_
+        expect(lambda: expect(1).fnord()).raises(AssertionError, r"Expect 1 sentinel")
+        expect(lambda: expect(1).not_fnord()).raises(AssertionError, r"Expect 1 not sentinel")
+    
+    def test_assertion_can_contain_unicode_message(self):
+        class local_expect(expect): pass
+        local_expect.fnord = lambda self: self._assert(False, "Fnörd")
+        expect(lambda: local_expect(1).fnord()).to_raise(AssertionError, r"^Expect 1 Fnörd$")
+        
+        local_expect.fnord = lambda self: self._assert(False, u"Fnörd")
+        expect(lambda: local_expect(1).fnord()).to_raise(AssertionError, r"^Expect 1 Fnörd$")
+    
+    def test_error_message_when_calling_non_existing_matcher_is_good(self):
         expect(lambda: expect('fnord').nonexisting_matcher()) \
             .to_raise(AssertionError, r"Tried to call non existing matcher 'nonexisting_matcher'")
     
-    def test_should_ensure_not_is_on_word_boundaries(self):
+    def test_not_negates_only_if_on_word_boundaries(self):
         expect(lambda: expect(True).nothing_that_negates.is_(True)).not_.to_raise()
         expect(lambda: expect(True).annotation.to.be(True)).not_.to_raise()
         expect(lambda: expect(True).an_not_ation.to.be(True)).to_raise()
         expect(lambda: expect(True).an_not.to.be(True)).to_raise()
     
-    def test_should_allow_custom_messages(self):
+    def test_can_specify_custom_message(self):
         def messaging(message):
             return lambda: expect(True, message=message).not_.to_be(True)
         expect(messaging('fnord')).to_raise(AssertionError, r"^fnord$")
@@ -499,28 +542,14 @@ class ExpectTest(TestCase):
         
         # TODO: allow partially formatted messages from expect.with_message
     
-    def test_should_allow_to_formulate_non_raising_expectations(self):
+    def test_can_return_error_instead_of_raising(self):
         # Idea: have a good api to check expectations without raising
         expect(expect(False, should_raise=False).to_be(False)).equals((True, ""))
         expect(expect(False, should_raise=False).not_to.be(True)).equals((True, ""))
         expect(expect.returning(False).to_be(False)).equals((True, ""))
         expect(expect.returning(False).to_be(True)).to_equal((False, "Expect False to be True"))
     
-    def test_should_hide_double_underscore_alternative_names_from_tracebacks(self):
-        assertion = None
-        try:
-            expect(3) != 3
-        except AssertionError as a:
-            assertion = a
-        
-        expect(assertion) != None
-        import traceback
-        traceback = '\n'.join(traceback.format_tb(sys.exc_info()[2]))
-        expect(traceback).not_to.contain('_assert')
-        # Would like this one too, but is only hidden in py.test
-        # expect(traceback).not_to.contain('self(')
-    
-    def test_should_allow_not_variety_of_every_matcher_directly(self):
+    def test_not_in_path_inverts_every_matcher(self):
         expect(3).to_be(3)
         expect(3).not_to_be(2)
         expect(lambda: expect(3).not_to_be(3)) \
@@ -530,9 +559,27 @@ class ExpectTest(TestCase):
         raising = lambda: expect(lambda: 1 / 0).not_to_raise(ZeroDivisionError)
         expect(raising).to_raise(AssertionError, "division.* by zero")
     
-    def _test_should_give_good_error_message_when_missing_argument_to_expect(self):
+    def test_missing_argument_to_expect_raises_with_good_error_message(self):
+        expect(lambda: expect()).to_raise(TypeError)
+    
+    def _test_stacktrace_contains_matcher_as_top_level_entry(self):
         pass
     
+    def _test_stacktrace_does_not_contain_internal_methods(self):
+        pass
+    
+    def test_hides_double_underscore_alternative_names_from_tracebacks(self):
+        assertion = None
+        try:
+            expect(3) != 3
+        except AssertionError as a:
+            assertion = a
+        
+        expect(assertion) != None
+        import traceback
+        traceback = '\n'.join(traceback.format_tb(sys.exc_info()[2]))
+        expect(traceback).not_to.contain('__ne__')
+    
     # Matchers ##########################################################################################
     
     # REFACT: rename tests to use the canonical name
@@ -647,9 +694,16 @@ class ExpectTest(TestCase):
         expect([23,42]).not_to.contain(7)
         expect(dict(foo='bar')).includes('foo')
         expect([1,2,3,4]).includes(2,3)
+        expect(dict(foo='bar')).has_key('foo')
         
         expect(lambda: expect([1,2]).to.contain(3)).to_raise(AssertionError, r"Expect \[1, 2] to include 3")
         expect(lambda: expect([1,2]).to_include(2,3)).to_raise(AssertionError, r"Expect \[1, 2] to include 3")
+        
+        native_python_error_message = r"includes\(\) missing 1 required positional argument: 'needle'"
+        if sys.version < '3':
+            native_python_error_message = r"includes\(\) takes at least 2 arguments \(1 given\)"
+        
+        expect(lambda: expect((1,2)).includes()).to_raise(TypeError, native_python_error_message)
     
     def test_has_subdict(self):
         expect(dict()).to_have.subdict()
@@ -777,6 +831,7 @@ class ExpectTest(TestCase):
         
         expect(lambda: expect([42]).to_have.length(23)) \
             .to_raise(AssertionError, r"Expect \[42\] to have length 23, but found length 1")
+        # TODO: should assert that out supports __len__
     
     def test_greater_than(self):
         expect(3).is_greater_than(1)
@@ -818,6 +873,18 @@ class ExpectTest(TestCase):
         
         expect(lambda: expect(10).is_.close_to(2, 3)).to_raise(AssertionError, "Expect 10 to be close to 2 with max delta 3")
     
+    def test_hasattr(self):
+        expect(dict()).hasattr('items')
+        expect(object).not_.hasattr('fnord')
+        
+        expect(lambda: expect(object).hasattr('fnord')).to_raise(AssertionError, "Expect <(?:class|type) 'object'> to have attribute 'fnord'")
+    
+    def test_is_subclass_of(self):
+        expect(dict).is_subclass_of(object)
+        expect(dict).is_not.subclass_of(int)
+        
+        expect(lambda: expect(dict).subclass_of(int)).to_raise(AssertionError, "Expect <(?:class|type) 'dict'> to be subclass of <(?:class|type) 'int'>")
+    
     def _test_increases_by(sel):
         # Not sure what the right syntax for this should be
         # increase_by, increases_by
